﻿using System;
using System.Runtime.InteropServices;
using System.Collections.Generic;
using System.Text;
using System.Drawing;
using SlimDX;

using DX11 = SlimDX.Direct3D11;
using DXGI = SlimDX.DXGI;
using D3DCompiler = SlimDX.D3DCompiler;

namespace TutorialSeries.DirectX10.Chapter2
{
    public class SimpleTriangle
    {
        private DX11.Buffer m_vertexBuffer;
        private DX11.InputLayout m_vertexLayout;
        private DX11.Effect m_effect;
        private DX11.EffectTechnique m_effectTechnique;
        private DX11.EffectPass m_effectPass;

        public void LoadResources(DX11.Device device)
        {
            Vertex[] vertices = new Vertex[]
            {
                new Vertex(new Vector4(-0.8f, -0.8f, 0f, 1f), Color.Red.ToArgb()),
                new Vertex(new Vector4(0f, 0.8f, 0f, 1f), Color.Green.ToArgb()),
                new Vertex(new Vector4(0.8f, -0.8f, 0f, 1f), Color.Blue.ToArgb())
            };

            DataStream outStream = new DataStream(3 * Marshal.SizeOf(typeof(Vertex)), true, true);
            outStream.WriteRange(vertices);
            outStream.Position = 0;

            DX11.BufferDescription bufferDescription = new DX11.BufferDescription();
            bufferDescription.BindFlags = DX11.BindFlags.VertexBuffer;
            bufferDescription.CpuAccessFlags = DX11.CpuAccessFlags.None;
            bufferDescription.OptionFlags = DX11.ResourceOptionFlags.None;
            bufferDescription.SizeInBytes = 3 * Marshal.SizeOf(typeof(Vertex));
            bufferDescription.Usage = DX11.ResourceUsage.Default;

            m_vertexBuffer = new DX11.Buffer(device, outStream, bufferDescription);
            outStream.Close();

            D3DCompiler.ShaderBytecode shaderByteCode =
                D3DCompiler.ShaderBytecode.CompileFromFile(
                    "Shaders/SimpleRendering.fx", "fx_5_0",
                    D3DCompiler.ShaderFlags.None, D3DCompiler.EffectFlags.None);
            m_effect = new DX11.Effect(
                device, shaderByteCode);
            m_effectTechnique = m_effect.GetTechniqueByIndex(0);
            m_effectPass = m_effectTechnique.GetPassByIndex(0);

            DX11.InputElement[] inputElements = new DX11.InputElement[]
            {
                new DX11.InputElement("POSITION",0,DXGI.Format.R32G32B32A32_Float,0,0),
                new DX11.InputElement("COLOR",0,DXGI.Format.R8G8B8A8_UNorm,16,0)
            };

            m_vertexLayout = new DX11.InputLayout(device, inputElements, m_effectPass.Description.Signature);
        }

        public void Render(DX11.Device device)
        {
            DX11.DeviceContext deviceContext = device.ImmediateContext;

            deviceContext.InputAssembler.InputLayout = m_vertexLayout;
            deviceContext.InputAssembler.PrimitiveTopology = DX11.PrimitiveTopology.TriangleList;
            deviceContext.InputAssembler.SetVertexBuffers(0, new DX11.VertexBufferBinding(m_vertexBuffer, Marshal.SizeOf(typeof(Vertex)), 0));

            m_effectPass.Apply(deviceContext);
            deviceContext.Draw(3, 0);
        }
    }
}
