﻿using System;
using System.Runtime.InteropServices;
using System.Collections.Generic;
using System.Text;
using System.Drawing;
using SlimDX;

using DX11 = SlimDX.Direct3D11;
using DXGI = SlimDX.DXGI;
using D3DCompiler = SlimDX.D3DCompiler;

namespace TutorialSeries.DirectX10.Chapter4
{
    public class SimpleBox
    {
        private DX11.Buffer m_vertexBuffer;
        private DX11.Buffer m_indexBuffer;
        private DX11.InputLayout m_vertexLayout;
        private DX11.Effect m_effect;
        private DX11.EffectTechnique m_effectTechnique;
        private DX11.EffectPass m_effectPass;
        private DX11.EffectMatrixVariable m_transformVariable;
        private DX11.Texture2D m_texture;
        private DX11.EffectResourceVariable m_resourceVariable;
        private DX11.ShaderResourceView m_textureView;

        public void LoadResources(DX11.Device device)
        {
            Vertex[] vertices = new Vertex[]
            {
                new Vertex(new Vector3(-1, -1, -1), new Vector2(0f,0f)),
                new Vertex(new Vector3(1, -1, -1), new Vector2(1f, 0f)),
                new Vertex(new Vector3(1, -1, 1), new Vector2(0f, 0f)),
                new Vertex(new Vector3(-1, -1, 1), new Vector2(1f, 0f)),
                new Vertex(new Vector3(-1, 1, -1), new Vector2(0f, 1f)),
                new Vertex(new Vector3(1,1,-1), new Vector2(1f, 1f)),
                new Vertex(new Vector3(1,1,1), new Vector2(0f, 1f)),
                new Vertex(new Vector3(-1, 1,1), new Vector2(1f, 1f)),
                
                new Vertex(new Vector3(-1, -1, -1), new Vector2(0f,0f)),
                new Vertex(new Vector3(1, -1, -1), new Vector2(1f, 0f)),
                new Vertex(new Vector3(1, -1, 1), new Vector2(1f, 1f)),
                new Vertex(new Vector3(-1, -1, 1), new Vector2(0f, 1f)),
                new Vertex(new Vector3(-1, 1, -1), new Vector2(0f, 0f)),
                new Vertex(new Vector3(1,1,-1), new Vector2(1f, 0f)),
                new Vertex(new Vector3(1,1,1), new Vector2(1f, 1f)),
                new Vertex(new Vector3(-1, 1,1), new Vector2(0f, 1f))
            };

            short[] indices = new short[]
            {
                4,1,0,4,5,1,
                5,2,1,5,6,2,
                6,3,2,6,7,3,
                15,8,11,15,12,8,
                15,13,12,15,14,13,
                10,11,8,10,8,9
            };

            DataStream outStream = new DataStream(16 * Marshal.SizeOf(typeof(Vertex)), true, true);
            DataStream outStreamIndex = new DataStream(36 * Marshal.SizeOf(typeof(short)), true, true);

            for (int loop = 0; loop < vertices.Length; loop++)
            {
                outStream.Write<Vertex>(vertices[loop]);
            }

            for(int loop=0 ; loop<indices.Length ; loop++)
            {
                outStreamIndex.Write<short>(indices[loop]);
            }

            outStream.Position = 0;
            outStreamIndex.Position = 0;
            
            DX11.BufferDescription bufferDescription = new DX11.BufferDescription();
            bufferDescription.BindFlags = DX11.BindFlags.VertexBuffer;
            bufferDescription.CpuAccessFlags = DX11.CpuAccessFlags.None;
            bufferDescription.OptionFlags = DX11.ResourceOptionFlags.None;
            bufferDescription.SizeInBytes = 16 * Marshal.SizeOf(typeof(Vertex));
            bufferDescription.Usage = DX11.ResourceUsage.Default;

            DX11.BufferDescription bufferDescriptionIndex = new DX11.BufferDescription();
            bufferDescriptionIndex.BindFlags = DX11.BindFlags.IndexBuffer;
            bufferDescriptionIndex.CpuAccessFlags = DX11.CpuAccessFlags.None;
            bufferDescriptionIndex.OptionFlags = DX11.ResourceOptionFlags.None;
            bufferDescriptionIndex.SizeInBytes = 36 * Marshal.SizeOf(typeof(short));
            bufferDescriptionIndex.Usage = DX11.ResourceUsage.Default;

            m_vertexBuffer = new DX11.Buffer(device, outStream, bufferDescription);
            m_indexBuffer = new DX11.Buffer(device, outStreamIndex, bufferDescriptionIndex);
            outStream.Close();
            outStreamIndex.Close();

            D3DCompiler.ShaderBytecode shaderByteCode =
               D3DCompiler.ShaderBytecode.CompileFromFile(
                   "Shaders/SimpleRendering.fx", "fx_5_0",
                   D3DCompiler.ShaderFlags.None, D3DCompiler.EffectFlags.None);
            m_effect = new DX11.Effect(
                device, shaderByteCode);
            m_effectTechnique = m_effect.GetTechniqueByIndex(0);
            m_effectPass = m_effectTechnique.GetPassByIndex(0);
            m_transformVariable = m_effect.GetVariableByName("WorldViewProj").AsMatrix();
            m_resourceVariable = m_effect.GetVariableByName("Texture").AsResource();

            DX11.InputElement[] inputElements = new DX11.InputElement[]
            {
                new DX11.InputElement("POSITION",0,SlimDX.DXGI.Format.R32G32B32_Float,0,0),
                new DX11.InputElement("TEXCOORD",0,SlimDX.DXGI.Format.R32G32_Float,12,0)
            };
          
            m_vertexLayout = new DX11.InputLayout(device, inputElements, m_effectPass.Description.Signature);

            m_texture = DX11.Texture2D.FromFile(device, "Textures/Texture.png");
            m_textureView = new DX11.ShaderResourceView(device, m_texture);
        }

        public void Render(DX11.Device device, Matrix world, Matrix viewProj)
        {
            DX11.DeviceContext deviceContext = device.ImmediateContext;

            deviceContext.InputAssembler.InputLayout = m_vertexLayout;
            deviceContext.InputAssembler.PrimitiveTopology = DX11.PrimitiveTopology.TriangleList;
            deviceContext.InputAssembler.SetIndexBuffer(m_indexBuffer, DXGI.Format.R16_UInt, 0);
            deviceContext.InputAssembler.SetVertexBuffers(0, new DX11.VertexBufferBinding(m_vertexBuffer, Marshal.SizeOf(typeof(Vertex)), 0));
           
            m_transformVariable.SetMatrix(world * viewProj);
            m_resourceVariable.SetResource(m_textureView);

            m_effectPass.Apply(deviceContext);
            deviceContext.DrawIndexed(36, 0, 0);
        }
    }
}
